# -*- coding: utf-8 -*-
#
# This file is part of fedrepo_req.
# Copyright © 2017 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
"""
The fedrepo-req script
"""
from __future__ import absolute_import
import click
import json
from fedrepo_req import (
    NAMESPACE_CHOICES, MONITOR_CHOICES, BUGZILLA_CLIENT, STREAM_NAME_ERROR)
from fedrepo_req.utils import (
    sla_list_to_dict, verify_slas, is_valid_standard_branch,
    valid_project_name, valid_module_stream_name)
from fedrepo_req.exceptions import ValidationError
import fedrepo_req.pagure

SLA_HELP = ('The service levels tied to the branch. This must be in the '
            'format of "sl_name:2020-12-01"')
EXCEPTION_HELP = (
    'The package is an exception to the regular package review process '
    '(specifically, it does not require a Bugzilla bug).')


@click.command(context_settings={'help_option_names': ['-h', '--help']})
@click.option('--exception', is_flag=True, help=EXCEPTION_HELP, default=False)
@click.option('--description', '-d', help='The package\'s description.')
@click.option('--monitor', '-m', type=click.Choice(MONITOR_CHOICES),
              default=MONITOR_CHOICES[2],
              help='Monitoring type for the package.')
@click.option('--namespace', '-n', type=click.Choice(NAMESPACE_CHOICES),
              default='rpms', help='The repo\'s namespace.')
@click.option('--summary', '-s',
              help='Override the package\'s summary from the Bugzilla bug.')
@click.option('--ticket', '-t',
              help='Bugzilla bug ID of the package review request.')
@click.option('--upstreamurl', '-u', help='The package\'s upstream URL.')
@click.option('--sl', multiple=True, help=SLA_HELP)
@click.argument('repo')
@click.argument('branch', default='master')
def cli(branch, repo, sl, upstreamurl, ticket, summary, namespace, monitor,
        description, exception):
    """
    Request new repositories on src.fedoraproject.org. If a branch is
    specified, it will create it in addition to the master branch in PDC.

    Examples:

        fedrepo-req nethack -t 12345

        fedrepo-req nethack epel7 -t 12345

        fedrepo-req nethack lts -t 12345 --sl security_fixes:2022-12-01
        --sl bug_fixes:2022-12-01

        fedrepo-req nethack --exception
    """
    # This name makes more sense for the contents of sl, where as --sl makes
    # more sense from the cli perspective
    slas = sl
    branch = branch.strip()
    repo = repo.strip()
    if not ticket and not exception:
        raise click.ClickException(
            'A Bugzilla bug is required on new repository requests. Use the '
            '"-t" or "--ticket" option.')
    if not valid_project_name(repo):
        raise click.ClickException(
            'The repository name "{0}" is invalid. It must be at least two '
            'characters long with only letters, numbers, hyphens, '
            'underscores, plus signs, and/or periods. Please note that the '
            'project cannot start with a period or a plus sign.'.format(repo))

    module_ns = ['modules', 'test-modules']
    if namespace in module_ns and not valid_module_stream_name(branch):
        raise click.ClickException(STREAM_NAME_ERROR)

    # If SLAs are not provided and it isn't a standard branch, then raise an
    # error
    if not slas and not is_valid_standard_branch(branch):
        raise click.ClickException(
            'You must provide SLs for non-standard branches')
    # If SLAs were provided, no matter what kind of branch it is, verify them
    elif slas:
        try:
            sla_dict = sla_list_to_dict(list(slas))
            verify_slas(branch, sla_dict)
        except ValidationError as error:
            raise click.ClickException(str(error))

    summary_from_bug = ''
    if ticket:
        try:
            BUGZILLA_CLIENT.use_creds = False
            bug = BUGZILLA_CLIENT.get_valid_review_bug(
                ticket, repo, branch, require_auth=False, namespace=namespace)
        except ValidationError as error:
            raise click.ClickException(str(error))

        summary_from_bug = bug.summary.split(' - ', 1)[1].strip()

    ticket_body = {
        'action': 'new_repo',
        'branch': branch,
        'bug_id': ticket or '',
        'description': description or '',
        'exception': exception,
        'monitor': monitor,
        'namespace': namespace,
        'repo': repo,
        'summary': summary or summary_from_bug,
        'upstreamurl': upstreamurl or ''
    }
    if slas:
        ticket_body['sls'] = sla_dict
    ticket_body = json.dumps(ticket_body, indent=True)
    ticket_body = '```\n{0}\n```'.format(ticket_body)
    ticket_title = 'New Repo for "{0}/{1}"'.format(namespace, repo)
    click.echo(fedrepo_req.pagure.new_issue(ticket_title, ticket_body))


if __name__ == '__main__':
    cli()
