# -*- coding: utf-8 -*-
#
# This file is part of fedpkg-req.
# Copyright © 2017 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
"""
The fedrepo-req-branch script
"""
from __future__ import absolute_import
import click
import json

from fedrepo_req.utils import (
    sla_list_to_dict, verify_slas, is_valid_standard_branch)
from fedrepo_req.exceptions import ValidationError
import fedrepo_req.pagure
from fedrepo_req import NAMESPACE_CHOICES


SLA_HELP = ('The service levels tied to the branch. This must be in the '
            'format of "sl_name:2020-12-01"')


@click.command(context_settings={'help_option_names': ['-h', '--help']})
@click.option('--git-branch/--no-git-branch', default=True,
              help='Create the new branch in git. This defaults to on.')
@click.option('--namespace', '-n', type=click.Choice(NAMESPACE_CHOICES),
              default='rpms', help='The repo\'s namespace.')
@click.option('--sl', multiple=True, help=SLA_HELP)
@click.argument('repo')
@click.argument('branch')
def cli(branch, repo, sl, namespace, git_branch):
    """
    Request new branches on src.fedoraproject.org repositories. This adds a
    branch in git and an entry in PDC, which is required to build your
    package.

    Examples:

        fedrepo-req-branch nethack epel7

        fedrepo-req-branch nethack lts --sl security_fixes:2022-12-01
        --sl bug_fixes:2022-12-01
    """
    # This name makes more sense for the contents of sla, where as --sla makes
    # more sense from the cli perspective
    slas = sl
    branch = branch.strip()
    repo = repo.strip()

    if namespace in ['modules', 'test-modules'] and '-' in branch:
        raise click.ClickException('Dashes are not allowed in module branches')

    # If SLAs are not provided and it isn't a standard branch, then raise an
    # error
    if not slas and not is_valid_standard_branch(branch):
        raise click.ClickException(
            'You must provide SLs for non-standard branches')

    # If SLAs were provided, no matter what kind of branch it is, verify them
    elif slas:
        try:
            sla_dict = sla_list_to_dict(list(slas))
            verify_slas(branch, sla_dict)
        except ValidationError as error:
            raise click.ClickException(str(error))

    ticket_body = {
        'action': 'new_branch',
        'branch': branch,
        'namespace': namespace,
        'repo': repo,
        'create_git_branch': git_branch
    }
    if slas:
        ticket_body['sls'] = sla_dict
    ticket_body = json.dumps(ticket_body, indent=True)
    ticket_body = '```\n{0}\n```'.format(ticket_body)
    ticket_title = ('New Branch "{0}" for "{1}/{2}"'
                    .format(branch, namespace, repo))
    click.echo(fedrepo_req.pagure.new_issue(ticket_title, ticket_body))


if __name__ == '__main__':  # pragma: no cover
    cli()  # pragma: no cover
